/*

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Arlips compiler v. 3.1           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

File prin.c, main function
April 2006

Copyright (C) 2006 Emilio Vivancos Rubio

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              Contact                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Author:              Emilio Vivancos Rubio  vivancos@dsic.upv.es
Developer Arlips v2: Mario Gonzlez Gimnez mgonzale@dsic.upv.es
Developer Arlips v3: Carlos Garca Montoro  cargarcia@dsic.upv.es

Departamento de Sistemas Informticos y Computacin
Universidad Politcnica de Valencia
Camino de Vera s/n
46022 Valencia
Spain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "alex.h"
#include "tds.h"
#include "sinsem.h"
#include "errores.h"
//#include "es.h"

/***************************************************************************/
/********* VARIABLES *********/


char fich[80];          /* nombre fichero de entrada */
char fich_dest[80];     /* nombre fichero de destino */
char nombre_sp[80];     /* nombre del sistema de produccion */

extern int estrategia; /* estrategia de control del sistema de produccion */
extern int debug;      /* indica si se genera codigo debug */
extern int noprints;
extern int nomake;
extern int nomain;
extern int artis_mode;
extern int log_;
int verbose;   

extern char prefijo[9];  /* sufijo aadido a todos los identificadores generados */
                        /* cuando no se genera funcion main */ 

void mensaje_uso()
{
  printf("                           Arlips compiler v. 3.1\n");
   printf("\n");
   printf("Sintax:\n");
   printf("    arlips <file_name>[.arl] [-e:depth|width] [-debug] [-noprints]\n");
   printf("                             [-nomain] [-nomake] [-artis]\n"); 
   printf("                             [-input 'dir'] [-output 'dir'] [-v] [-log]\n");
   printf("OPTIONS:\n");
   printf(" -e  : Indicates the control strategy to be used\n");
   printf("          depth -> depth: LIFO (defect value)\n");
   printf("          width  -> width: FIFO \n");
/*   printf(" -ag : permite establecer el fichero que contendra la agenda del sistema de \n");
   printf("       produccion (por defecto se utiliza el fichero agenda_.c y agenda_.h)\n");
   printf("\n");
*/
   printf(" -debug : Includes debug info in the gererated code.\n");
   printf(" -noprints : Does not translate the PRINTS commands of Arlips.\n");
   printf(" -nomain : Does not generate the main() function.\n");
   printf(" -nomake : Does not generate the Makefile.\n");
   printf(" -artis  : Activates explicitly the artis compatibility of the generated code\n");
   printf(" -input  'dir': Sets the input directory to 'dir'.\n");
   printf("                The default input directory is the current directory.\n");
   printf(" -output 'dir': Sets the output directory of generated code to 'dir'.\n");
   printf("                The default output directory is the input directory.\n");
   printf(" -v      : Activates the verbose mode.\n");
   printf(" -log    : Activates the log mode.\n");
   printf("\n");
 }



/****************************************************************************/

/************************   PROGRAMA   PRINCIPAL   **************************/

int main(int argc, char *argv[]) {

  char * cadaux;
  char * cadext=".c";
  char * cadext2=".arl";
  char * estrprof = "E:DEPTH";
  char * estranch = "E:WIDTH";
  char * caddebug = "DEBUG";
  char * cadnomain = "NOMAIN";
  char * cadnomake = "NOMAKE";
  char * cadnoprints = "NOPRINTS";
  char * cad_output = "OUTPUT";
  char * cad_input = "INPUT";
  char * cadverbose = "V";
  char *cad_log = "LOG";
  char *cad_artis = "ARTIS";
  extern char *output_path;
  extern char *input_path;
  char error_analisis;
  int error_analex;        /* error producido al abrir el fichero en el */
                           /* analizador lexico */
  //int i;
  int cont;
  int error_arg = FALSE;
  int narg;

  algunerror=FALSE;
  input_path = NULL;
  output_path = NULL;

  /* NOTA: al abrir el fichero con O_RDONLY, el programa me sirve tanto */
  /* para UNIX como para DOS, ya que en dos, al abrirlo de esta manera */
  /* toma la opcion por defecto de convertir el retorno de carro a un  */
  /* solo caracter (toma la opcion de O_TEXT), tal y como es en UNIX */

  /* Si abrimos el fichero con O_RDONLY | O_BINARY, entonces el fichero */
  /* se lee como una tira de bytes donde el retorno de carro ya no sera */
  /* el mismo cuando estemos en DOS o en UNIX */


  if(argc>1){
    /* el primer argumento debe ser el fichero a procesar */
    
    if (strchr(argv[1], '/') != NULL) {
      fprintf(stderr, "\nERROR in the file_name argument.\n");
      fprintf(stderr, "Use the -input option to change the current source path\n");
      error_arg = TRUE;
    }
  
    strcpy(fich,argv[1]);
    strcpy(nombre_sp,fich);
    cadaux = strrchr(fich,'.');
    /* si el parametro pasado no tiene extension se le aade una por defecto */
    if (cadaux == NULL) {
      strcat(fich,cadext2);
    }
    
    /* ahora se procesan el resto de los argumentos */
    narg = 2;  /* se empieza en el segundo argumento */
    estrategia = profundidad;
    prefijo[0]='\0';
    //error_arg = FALSE;
    verbose = FALSE;
    
    while (narg<argc) {
      /* se convierte en mayusculas */
      for(cont=0; cont<strlen(argv[narg]); cont++) 
	argv[narg][cont] = toupper(argv[narg][cont]);
      if ((argv[narg][0]=='/') || (argv[narg][0]=='-')) {
	switch (argv[narg][1]) {
	case 'A': //modo artis
	  if (!strcmp(&argv[narg][1], cad_artis)) {
	    artis_mode = TRUE;
	    nomain = TRUE;
	  }
	  else {
	    error_arg = TRUE;
	  }
	  break;	    

	case 'E' : /* estrategia */
	  if (!strcmp(&argv[narg][1],estrprof)) { /* estrategia en profundidad */
	    estrategia = profundidad;
	  }
	  else
	    if (!strcmp(&argv[narg][1],estranch)) { /* estrategia en anchura */
	      estrategia = anchura;
	    } 
	    else
	      error_arg = TRUE;
	  break;
	case 'D' : /* Genera info para la depuracion */
	  if (!strcmp(&argv[narg][1],caddebug)) {
	    debug=TRUE;
	  } 
	  else
	    error_arg = TRUE;
	  break;
	case 'N' : /* No genera printfs */
	  if (!strcmp(&argv[narg][1],cadnoprints))
	    noprints = TRUE;
	  else
	    if (!strcmp(&argv[narg][1],cadnomain)) {
	      nomain = TRUE;
	    } 
	    else
	      if (!strcmp(&argv[narg][1],cadnomake))
		nomake = TRUE;
	      else
		error_arg = TRUE;
	  break;
	case 'V':
	  if (!strcmp(&argv[narg][1],cadverbose))
	    verbose = TRUE;
	  else
	    error_arg = TRUE;
	  break;
	  
	case 'I':
	  if (!strcmp(&argv[narg][1], cad_input)) {
	    narg++;
	    if (narg < argc) {
	      input_path = argv[narg];
	    }
	    else
	      error_arg = TRUE;
	  }
	  else
	    error_arg = TRUE;
	  break;
	  
	case 'O':
	  if (!strcmp(&argv[narg][1], cad_output)) {
	    narg++;
	    if (narg < argc) {
	      output_path = argv[narg];
	    }
	    else 
	      error_arg = TRUE;
	  }
	  else 
	    error_arg = TRUE;
	  break;

	case 'L':
	  if (!strcmp(&argv[narg][1], cad_log)) {
	  	log_ = TRUE;
	  }
	  else {
	  	error_arg = TRUE;
	  }
	  break;

	default:
	  error_arg = TRUE;
	  break;
	}
      }
      else
	error_arg = TRUE;
      narg++;
    }
    
    if (error_arg) {
      printf("\n");
      printf("ERROR: Uso incorrecto de argumentos\n\n");
      mensaje_uso();
      exit(-1);
    }
    else {
      if (output_path == NULL && input_path != NULL) {
	output_path = input_path;
      }
    }
    
    if (verbose) printf("Inicializando Analizador Lexico...\n");
    error_analex=Inicializa_Analizador_Lexico(fich);
    if (!error_analex) {
      obtsym();
      if (verbose)
	printf("\nIniciando Analisis Sintactico/Semantico...\n");
      programa(&error_analisis,verbose);
      if (!error_analisis && !algunerror) {
	if (verbose)
	  printf("\nProcesando condiciones para la deteccion de condiciones repetidas...\n");
	Detecta_cond_repetidas();
	/* se aade la extension .c al fichero pasado como parametro */
	strcpy(fich_dest,argv[1]);
	cadaux = strchr(fich_dest,'.');
	if (cadaux!=0)
	  *cadaux='\0';
	strcpy(nombre_sp,fich_dest);
	strcat(fich_dest,cadext);
	if (verbose)
	  printf("\nGenerando codigo C...\n");
	Genera_fuente(fich_dest,estrategia,nombre_sp,verbose);
      } 
      else
        printf("Se produjeron errores...\n");
      Cierra_Analizador_Lexico();
    }
  }
  else
    mensaje_uso();
}

