/*

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Arlips compiler v. 3.1           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

File tipos.h, types used by the compiler
April 2006

Copyright (C) 2006 Emilio Vivancos Rubio

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              Contact                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Author:              Emilio Vivancos Rubio  vivancos@dsic.upv.es
Developer Arlips v2: Mario Gonzlez Gimnez mgonzale@dsic.upv.es
Developer Arlips v3: Carlos Garca Montoro  cargarcia@dsic.upv.es

Departamento de Sistemas Informticos y Computacin
Universidad Politcnica de Valencia
Camino de Vera s/n
46022 Valencia
Spain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*/

#ifndef _TIPOS_
#define _TIPOS_

typedef union ENTREAL {
        int entero;
        float real;
} ENTREAL;

typedef struct LIS_NUM{
       ENTREAL num;
       struct LIS_NUM * sig;
       }LIS_NUM;

typedef struct LISTA_NUM {
       LIS_NUM *prim;
       LIS_NUM *ult;
       }LISTA_NUM;

typedef struct LISTA{
       char nombre[long_ident+1];
       struct LISTA * sig;
       }LISTA;   /* lista de identificadores */

typedef struct {            /* contenido de las variables de la tds */
	TIPOBASE *var;
	char flag;
       }VARIABLE;

/* la siguiente estructura sirve para acumular las condiciones afectadas */

typedef struct CONDACUM {
        int num_regla;
        int num_cond;
        int *param;     /* vector dinamico para almacenar el valor de los parametros */
        LIS_NUM *asig;  /* lista de asignaciones que provocaron la condicion acumulada */
        int num_param;
        struct CONDACUM * sig;
} CONDACUM;

typedef struct LISTA_CAMPOS {
  char nombre[long_ident+1];
  char tipo;
  char tbase_array;   //Tipo bsico si el tipo es complejo (por ejemplo, integer para un vector de enteros, 
  char inicializado;     /* booleano que indica si el campo esta inicializado */
  int lim_inf;
  int lim_sup;
  ENTREAL valor;
  LISTA_NUM *ld;                   /* conjunto de elementos si es de tipo lista */
  char instancia_de[long_ident+1]; /* clase a la que pertenece si tipo=definido */
  struct LISTA_CAMPOS * lc;        /* lista de campos si es de tipo definido */
  int tam_lista;
  LISTA * reglas;        /* reglas cuyas condiciones tienen alguna ref. a este campo */
  LIS_NUM * num_conds;     /* estan son las condiciones que hacen ref dentro de la regla */
  //  int com_kdm;  //Indica si el campo ser usado, y por tanto, si el del kdm, si requiere actualizacin
  int campo_buscado;  //Indica si el campo ha sido buscado. Si un campo se busca, es porque alguna regla lo usa.
                     //Cuando se usan variables patrn, estas marcan a los campos de todas las instancias.
  struct LISTA_CAMPOS * siguiente;
} LISTA_CAMPOS;

typedef struct REGTIPO{
     char nombre[long_ident+1];
     char tipo;
     struct REGTIPO * superclase;
     LISTA_CAMPOS * lc;
     int num_instancias;         /* numero de instancias  */
     int num_paternvars;         /* numero de paternvar   */
     int num_vars;               /* numero de variables   */

                              /* los siguientes campos son booleanos que sirven */
     int ya_declarada;        /* para llevar la cuenta de si una clase ha sido */
     int declarando_actualmente;  /* declarada por una que la usaba, o de si esta */
                              /* siendo declarada ya (para las estructuras recursivas) */
     } REGTIPO;

typedef struct REGTDS{
  char nombre[long_ident+1]; /* nombre de la variable */
  char tipo;                 /* tipo de la variable */
  int tbase_array;   //Tipo bsico si el tipo es complejo (por ejemplo, integer para un vector de enteros, 
  char instancia_de[long_ident+1]; /* clase a la que pertenece si tipo=definido */
  int from_kdm; //Especifca si la instancia ha sido generada por el parser del kdm
  char clvar;                /* clase de la var:                           */
  /*var,inputvar,patternvar,parametro,constante */
  int lim_inf;               /* si tipo=reg n1 es su tamao, sino */
  int lim_sup;               /* n1 y n2 son las cotas del tipo subrango*/
  LISTA_NUM *ld;             /* conjunto de elementos si es de tipo lista */
  int tam_lista;
  LISTA_CAMPOS *lc;          /* inicializacion de los campos si tipo=instancia */
  ENTREAL valor;             /* campo que contiene el puntero a la variable */
  char inicializado;         /* indica si es una instancia de tipo simple, si esta inicializada */
  union {
    int num_instancia;         /* numero de instancia si tipo=instancia */
    int num_paternvar;         /* numero de paternvar si tipo=paternvar */
  } cod;
  LISTA * reglas;            /* si la instancia es de tipo simple, estas son las reglas */
  /* que tienen una referencia a la instancia */
  LIS_NUM * num_conds;     /* estan son las condiciones que hacen ref dentro de la regla */
} REGTDS;

/***  elementos de las distintas tablas hash */

typedef struct EHTDS{
	 REGTDS etds;
	 struct EHTDS * sig;
       }EHTDS; /* elemento 'hash' de la tabla tds */

typedef struct EHTIPO{
	 REGTIPO etipo;
	 struct EHTIPO * sig;
       }EHTIPO; /* elemento 'hash' de la tabla tipo */

/*******  tipos para la tabla de reglas *******/

typedef struct EXPRESION {
  struct EXPRESION *indice_array; //apunta a la expresin que exprese el ndice del array
        int esoperando;     /* indica si este elemento es un operador o un operando */
        int escampo;        /* indica si es el campo de una instancia o paternvar */
        char tipooperador;  /* indica el tipo de operador: multiplicacion, mas, menos, division,existe,tail,head */
        char claseoperando; /* indica la clase del operando: instancia, patternvar, constante, ... */
        char tipooperando;  /* indica el tipo del operando: entero,real ... */
        char ident[long_ident+1];
        char *cad;
        ENTREAL valor;
        struct EXPRESION * sig;
} EXPRESION;

typedef struct LISTA_OBJ {
        char clvar;                       /* lista de instancias y pattern var */
        char tipo;                        /* tipo de la variable */
        char instancia_de[long_ident+1];  /* clase a la que pertenece */
        char nombre[long_ident+1];        /* nombre de la pattern var o instancia */
        char campo[long_ident+1];         /* campo de la pattern var o instancia */
        LISTA *campos;                    /* lista de campos, cuando se accede a campos que a su */
                                          /* vez son estructuras, se necesita una lista para seguir */
                                          /* la 'ruta de acceso' */
        char cuantificador;               /* indica como estan cuantificadas: existe o paratodo */
        union {
          int num_instancia;         /* numero de instancia si tipo=instancia */
          int num_paternvar;         /* numero de paternvar si tipo=paternvar */
        } cod;
        int por_ref;               /* si es un parametro, indica si es pasado por valor (FALSE) o por referencia (TRUE) */
        struct LISTA_OBJ * sig;
} LISTA_OBJ;

typedef struct LISTA_CONDIC {
        char oprel;
        int esta_negada;
        struct EXPRESION * lado_izq;
        struct EXPRESION * lado_der;

        int num_condic;            /* determina el numero de condicion dentro de la regla */
        LISTA_OBJ * lobj_cond;     /* se mantiene una lista de las vp y las instancias */
                                   /* que aparecen en las condiciones y en las acciones */
        LISTA *param;
        int num_vpe, num_vpu;

        int es_orig;               /* para las condiciones repetidas, hay una referencia */
        char regla_orig[long_ident+1];
        int cond_orig;

        struct LISTA_CONDIC * sig;
} LISTA_CONDIC;


typedef struct LISTA_ACCIONES{
       char tipo_accion;
       char ident[long_ident+1]; /* este identificador es: la patternvar, la */
                                 /* instancia o la variable sobre la que se actua */
       struct EXPRESION * campos; /* lista de campos, para cuando hay un tipo definido */
       char campo[long_ident+1];
       struct EXPRESION * expr;
       char tip_izq;       /* indica el tipo si es una asignacion */
       char tip_expr;      /* indica el tipo de la expresion si es una asignacion */
       char tip_lista_expr[30];   /* las listas de expresiones admiten hasta 30 */
                                  /* expresiones */
       struct LISTA_ACCIONES * sig;
}LISTA_ACCIONES;                 /* lista de acciones */


typedef struct ARB_COND{
        int tipo_nodo;  /* indica el tipo de nodo: AND, OR, o condicion */
        char oprel;
        int esta_negada;
        struct EXPRESION * lado_izq;
        struct EXPRESION * lado_der;
        struct ARB_COND *sig;
        struct ARB_COND *subarb_and;
        struct ARB_COND *subarb_or;
} ARB_COND;

typedef struct REGREGLA {
   char nombre[long_ident+1];
   char num_regla;            /* numero de regla */
   struct EXPRESION * expr_prio;   /* expresion de la prioridad */

   LISTA_CONDIC * lcond;
   LISTA_OBJ * lobj_cond;     /* se mantiene una lista de las vp y las instancias */
                              /* que aparecen en las condiciones y en las acciones */
   LISTA_ACCIONES * lacc;
   LISTA_OBJ * lobj_acc;
   LISTA *param;
   ARB_COND *condiciones;
   int num_vpe, num_vpu;

   int tiene_copias;          /* cuando una regla contiene ORs se generan otras */
   int num_copias;            /* numero de copias de la regla, sirve para controlar */
                              /* las copias con un codigo */
   int ya_generado;           /* booleano que controla si ya se ha generado la parte derecha de la regla */
   struct REGREGLA *copia_de; /* nuevas reglas, pero las reglas antiguas no se  */
                              /* borran de forma que siguen existiendo, pero no se */
                              /* procesan. Si tiene_copias=FALSE, entonces es una regla sin */
                              /* copias y debe ser procesada. */
}REGREGLA;

typedef struct EHREGLA{
	 REGREGLA eregla;
	 struct EHREGLA * sig;
}EHREGLA; /* elemento 'hash' de la tabla REGLA */


typedef struct LISTA_LISTA_CONDIC {
    LISTA_CONDIC * condicion;
    REGREGLA     * regla;
    struct LISTA_LISTA_CONDIC * sig;
} LISTA_LISTA_CONDIC;

/******  tipos para la tabla de funciones  ********/


typedef struct LISTA_PARAM{
     char nombre[long_ident+1];
     int por_ref;  /* indica si es por valor o por referencia */
     struct LISTA_PARAM *sig;
     REGTDS *ptro; /* puntero de un parametro a la tabla de simbolos */
} LISTA_PARAM;


typedef struct MODIFICACIONES {
/* esta estructura almacena las modificaciones de la BH que pueden producirse
 en las funciones */
    char nombre[long_ident+1];
    struct EXPRESION * campos; /* lista de campos, para cuando hay un tipo definido */
    char clase_obj;            /* puede ser una instancia o un parametro por referencia */
    char tip_izq;              /* indica el tipo del objeto modificado, incluyendo los campos */
    struct MODIFICACIONES *sig;
} MODIFICACIONES;

typedef struct {
     char nombre[long_ident+1];
     LISTA_PARAM *lp; /* lista de parametros de la funcion */
     LISTA_ACCIONES *lacc; /* lista de operaciones que se realizan en la */
                             /* funcion */

     char tipo;     /* tipo de funcion: externa o interna */
     char tipo_dev; /* tipo devuelto por la funcion */
     LISTA_OBJ * lobj_func; /* lista de objetos (instancias y vp) que aparecen */
                            /* en la funcion */

     CONDACUM * cond_ac;    /* condiciones acumuladas */
     int num_cond_ac;       /* numero de condiciones acumuladas */

     MODIFICACIONES * modif;
     /* los siguientes campos solo tendran valor en caso */
     /* de funciones externas */
     char * param;
     char nombre_ext[long_ident+1];

} REGFUNC;

typedef struct EHFUNC{
         REGFUNC efunc;
         struct EHFUNC * sig;
} EHFUNC; /* elemento 'hash' de la tabla func */

#endif
